﻿// Climate Framework for Uncertainty, Negotiation and Distribution
// Version 3.5
// (c) David Anthoff and Richard S.J. Tol
// http://www.fund-model.org
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Globalization;

namespace Fund
{
    public class NormalDistributionParameter : Parameter
    {
        private double m_value;
        private double m_bestGuess;
        private double m_standardDeviation;
        private bool m_useLowerBound, m_useUpperBound;
        private double m_lowerBound, m_upperBound;

        protected override double GetValue()
        {
            return m_value;
        }

        public NormalDistributionParameter(ParameterKey Key, string[] Arguments, Random rand)
            : base(Key, rand)
        {

            m_bestGuess = Convert.ToDouble(Arguments[0], CultureInfo.InvariantCulture);
            m_standardDeviation = Convert.ToDouble(Arguments[1], CultureInfo.InvariantCulture);

            if (Arguments.Length > 2)
            {
                if (Arguments[2].Trim().Length > 0)
                {
                    m_useLowerBound = true;
                    m_lowerBound = Convert.ToDouble(Arguments[2], CultureInfo.InvariantCulture);
                }
                else
                    m_useLowerBound = false;

                if (Arguments.Length > 3)
                {
                    if (Arguments[3].Trim().Length > 0)
                    {
                        m_useUpperBound = true;
                        m_upperBound = Convert.ToDouble(Arguments[3], CultureInfo.InvariantCulture);
                    }
                    else
                        m_useUpperBound = false;
                }
            }

            SetToBestGuessValue();
        }

        public NormalDistributionParameter(ParameterKey Key, double BestGuess, double StandardDeviation, Random rand)
            : base(Key, rand)
        {

            m_bestGuess = BestGuess;
            m_standardDeviation = StandardDeviation;
            m_useLowerBound = false;
            m_useUpperBound = false;
            SetToBestGuessValue();

        }

        public NormalDistributionParameter(ParameterKey Key, double BestGuess, double StandardDeviation, bool UseLowerBound, double LowerBound, bool UseUpperBound, double UpperBound, Random rand)
            : base(Key, rand)
        {
            m_bestGuess = BestGuess;
            m_standardDeviation = StandardDeviation;
            m_useLowerBound = UseLowerBound;
            m_useUpperBound = UseUpperBound;
            m_lowerBound = LowerBound;
            m_upperBound = UpperBound;
            SetToBestGuessValue();

        }

        public override void SetToBestGuessValue()
        {
            m_value = m_bestGuess;
        }

        public override void SetToRandomValue()
        {

            double i_helper1, i_helper2;

            LegacyFund28Math.GenNormal(m_bestGuess, m_standardDeviation, out i_helper1, out i_helper2, _rand);
            m_value = i_helper1;

            while ((m_useLowerBound && (m_value < m_lowerBound)) || (m_useUpperBound && (m_value > m_upperBound)))
            {
                LegacyFund28Math.GenNormal(m_bestGuess, m_standardDeviation, out i_helper1, out i_helper2, _rand);
                m_value = i_helper1;
            }
        }

        public double? LowerBound
        {
            get
            {
                if (m_useLowerBound)
                    return m_lowerBound;
                else
                    return null;
            }
        }

        public double? UpperBound
        {
            get
            {
                if (m_useUpperBound)
                    return m_upperBound;
                else
                    return null;
            }
        }

        public override double Mean { get { return m_bestGuess; } }
        public override double StdDev { get { return m_standardDeviation; } }
    }
}
