﻿// Climate Framework for Uncertainty, Negotiation and Distribution
// Version 3.5
// (c) David Anthoff and Richard S.J. Tol
// http://www.fund-model.org
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Fund
{
    public class RegionRegionParameterKey : ParameterKey
    {
        private string _region1 = String.Empty;
        private int _region1Id;
        private string _region2 = String.Empty;
        private int _region2Id;

        public RegionRegionParameterKey(string name, string region1, string region2)
            : base(name)
        {

            _region1 = region1;
            _region1Id = LegacyRegionNames.GetIdFromName(region1);
            _region2 = region2;
            _region2Id = LegacyRegionNames.GetIdFromName(region2);
        }

        public RegionRegionParameterKey(string name, int region1Id, int region2Id)
            : base(name)
        {
            _region1Id = region1Id;
            _region2Id = region2Id;
        }

        public override bool Equals(object obj)
        {
            if (obj == null) return false;
            if (Object.ReferenceEquals(this, obj)) return true;
            if (this.GetType() != obj.GetType()) return false;

            RegionRegionParameterKey castedObj = (RegionRegionParameterKey)obj;

            if (this._name.Equals(castedObj._name) && 
                this._region1Id.Equals(castedObj._region1Id) &&
                this._region2Id.Equals(castedObj._region2Id)) return true;

            return false;
        }

        public override int GetHashCode()
        {
            return _name.GetHashCode() ^ (_region1Id << 16) ^ (_region2Id<<24);
        }


        public override string ToString()
        {
            return String.Format("{0};{1};{2}", Name, Region1, Region2);
        }

        public string Region1
        { 
            get
            {
                if (_region1 == String.Empty)
                    _region1 = LegacyRegionNames.GetName(_region1Id);

                return _region1;
            } 
        }

        public int Region1Id { get { return _region1Id; } }

        public string Region2 
        { 
            get
            {
                if (_region2 == String.Empty)
                    _region2 = LegacyRegionNames.GetName(_region2Id);

                return _region2;
            } 
        }

        public int Region2Id { get { return _region2Id;}}
    }
}
