﻿// Climate Framework for Uncertainty, Negotiation and Distribution
// Version 3.5
// (c) David Anthoff and Richard S.J. Tol
// http://www.fund-model.org
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Fund
{
    public class YearRegionParameterKey : ParameterKey
    {
        private int _year;
        private string _region = String.Empty;
        private int _regionId;

        public YearRegionParameterKey(string name, int year, string region)
            : base(name)
        {
            _year = year;
            _region = region;
            _regionId = LegacyRegionNames.GetIdFromName(region);
        }

        public YearRegionParameterKey(string name, int year, int regionId)
            : base(name)
        {
            _year = year;
            _regionId = regionId;
        }

        public override bool Equals(object obj)
        {
            if (obj == null) return false;
            if (Object.ReferenceEquals(this, obj)) return true;
            if (this.GetType() != obj.GetType()) return false;

            YearRegionParameterKey castedObj = (YearRegionParameterKey)obj;

            if (this._name.Equals(castedObj._name) &&
                this._year.Equals(castedObj._year) &&
                this._regionId.Equals(castedObj._regionId)) return true;

            return false;
        }

        public override int GetHashCode()
        {
            return _name.GetHashCode() ^ _year ^ (_regionId << 16);
        }

        public override string ToString()
        {
            return String.Format("{0};{1};{2}", Name, Year + 1950, Region);
        }

        public int Year { get { return _year; } }
        public string Region
        {
            get 
            {
                if (_region == String.Empty)
                    _region = LegacyRegionNames.GetName(_regionId);
                return _region; 
            } 
        }
        public int RegionId { get { return _regionId; } }
    }
}
