// Climate Framework for Uncertainty, Negotiation and Distribution
// Version 3.5
// (c) David Anthoff and Richard S.J. Tol
// http://www.fund-model.org
using System;
using System.Collections.Generic;
using System.Text;
using System.Diagnostics;

namespace Esmf
{
    public class Variable1Dimensional<D1, T> : IVariable1Dimensional<D1, T>, IParameter1Dimensional<D1, T>
    {
        private Dictionary<D1, T> _values = new Dictionary<D1, T>();
        private ModelFields _parent;

        public Variable1Dimensional(ModelFields parent)
        {
            _parent = parent;
        }

        [Conditional("FUNDNANCHECK")]
        private void CheckForValidValue(T v)
        {
            if (v.GetType() == typeof(double))
            {
                double d = ((double)((object)v));

                if (double.IsNaN(d))
                    throw new ArgumentOutOfRangeException("NaN is not allowed as a value");
                else if (double.IsInfinity(d))
                    throw new ArgumentOutOfRangeException("Infinity is not allowed as a value");
            }
        }

        public T this[D1 index]
        {
            get
            {
                return _values[index];
            }
            set
            {
                CheckForValidValue(value);

                if (_values.ContainsKey(index))
                {
                    _values[index] = value;
                }
                else
                {
                    _values.Add(index, value);
                }
            }
        }

        #region IParameter1Dimensional<D1,T> Members

        T IParameter1Dimensional<D1, T>.this[D1 index]
        {
            get
            {
                return this[index];

            }
        }

        IEnumerable<Parameter1DimensionalMember<D1, T>> IParameter1Dimensional<D1, T>.GetEnumerator()
        {
            throw new NotImplementedException();
        }

        #endregion

        #region IVariable1Dimensional<D1,T> Members

        T IVariable1Dimensional<D1, T>.this[D1 index]
        {
            get
            {
                return this[index];
            }
            set
            {
                this[index] = value;
            }
        }

        IEnumerable<Parameter1DimensionalMember<D1, T>> IVariable1Dimensional<D1, T>.GetEnumerator()
        {
            return (this as IParameter1Dimensional<D1, T>).GetEnumerator();
        }

        #endregion

    }
}
