﻿// Climate Framework for Uncertainty, Negotiation and Distribution
// Version 3.5
// (c) David Anthoff and Richard S.J. Tol
// http://www.fund-model.org
using CommandLine;
using System.Threading;
using System.Configuration;
using System.Globalization;
using System.IO;
using System.Collections;
using System.Collections.Generic;
using System.Diagnostics;
using Esmf;
using System;

namespace Fund
{

    public static class ConsoleApp
    {

        public static void Main(string configurationFileName)
        {
            TMarginalRun lMarginalRun;

            TextWriter gDisaggregatedCsvFile = null;
            TextWriter gDisaggregatedYearCsvFile = null;
            TextWriter gDisaggregatedYearRegionCsvFile = null;
            TextWriter gDisaggregatedYearRegionWeightingschemeCsvFile = null;
            TextWriter gBGEResultsCSVFile = null;

            TextWriter lAggregateMarginalDamage = null;

            TextWriter lDimGasCsv = null;
            TextWriter lDimRegionCsv = null;
            TextWriter lDimSectorCsv = null;
            TextWriter lDimYearCsv = null;
            TextWriter lGlobalInputCsv = null;
            TextWriter lRegionInputCsv = null;
            TextWriter lYearInputCsv = null;
            TextWriter lRegionYearInputCsv = null;
            TextWriter lParameterIdCsv = null;
            string lYearStr;
            int lRuns;
            string lOutputPath;
            FundParameter lParam;
            Random lRandom;

            SimulationManager lSimulationManager;

            var lDefaultConfigurationFile = Path.Combine(Path.Combine(Path.GetDirectoryName(System.Reflection.Assembly.GetExecutingAssembly().Location), "Data"), "DefaultSimulation.xml");
            lSimulationManager = new SimulationManager(configurationFileName == null ? lDefaultConfigurationFile : configurationFileName);
            lSimulationManager.Load();

            lOutputPath = Path.Combine(Path.GetDirectoryName(System.Reflection.Assembly.GetExecutingAssembly().Location), "Data");

            if (!Directory.Exists(lOutputPath))
                Directory.CreateDirectory(lOutputPath);

            lRandom = GetNewRandom(lSimulationManager);

            lAggregateMarginalDamage = new StreamWriter(Path.Combine(lOutputPath, "Output - Aggregate marginal damage.csv"));
            gDisaggregatedYearCsvFile = new StreamWriter(Path.Combine(lOutputPath, "Output - Fact Year.csv"));
            var TempOutputFile = new StreamWriter(Path.Combine(lOutputPath, "Output - Temp.csv"));
            gBGEResultsCSVFile = new StreamWriter(Path.Combine(lOutputPath, "Output - BGE.csv"));

            if (lSimulationManager.OutputVerbal)
            {
                lAggregateMarginalDamage.Write("Scenario");
                lAggregateMarginalDamage.Write(";");
                lAggregateMarginalDamage.Write("Gas");
                lAggregateMarginalDamage.Write(";");
                lAggregateMarginalDamage.Write("Emissionyear");
                lAggregateMarginalDamage.Write(";");
                lAggregateMarginalDamage.Write("Run");
                lAggregateMarginalDamage.Write(";");
                lAggregateMarginalDamage.Write("Weightingscheme");
                lAggregateMarginalDamage.Write(";");
                lAggregateMarginalDamage.Write("Marginal damage");
                lAggregateMarginalDamage.WriteLine();
            }

            if (lSimulationManager.Runs.Exists((Run r) => r.OutputDisaggregatedData))
            {
                gDisaggregatedCsvFile = new StreamWriter(Path.Combine(lOutputPath, "Output - Fact YearRegionSectorWeightingscheme.csv"));
                gDisaggregatedYearRegionCsvFile = new StreamWriter(Path.Combine(lOutputPath, "Output - Fact YearRegionOutput.csv"));
                gDisaggregatedYearRegionWeightingschemeCsvFile = new StreamWriter(Path.Combine(lOutputPath, "Output - Fact YearRegionWeightingschemeOutput.csv"));

                if (lSimulationManager.OutputVerbal)
                {
                    gDisaggregatedCsvFile.Write("Scenario");
                    gDisaggregatedCsvFile.Write(";");
                    gDisaggregatedCsvFile.Write("Run");
                    gDisaggregatedCsvFile.Write(";");
                    gDisaggregatedCsvFile.Write("Marginal Gas");
                    gDisaggregatedCsvFile.Write(";");
                    gDisaggregatedCsvFile.Write("Marginal Emission Year");
                    gDisaggregatedCsvFile.Write(";");
                    gDisaggregatedCsvFile.Write("Year");
                    gDisaggregatedCsvFile.Write(";");
                    gDisaggregatedCsvFile.Write("Region");
                    gDisaggregatedCsvFile.Write(";");
                    gDisaggregatedCsvFile.Write("Sector");
                    gDisaggregatedCsvFile.Write(";");
                    gDisaggregatedCsvFile.Write("Weightingscheme");
                    gDisaggregatedCsvFile.Write(";");
                    gDisaggregatedCsvFile.Write("Damage");
                    gDisaggregatedCsvFile.WriteLine();
                }
            }

            lDimGasCsv = new StreamWriter(Path.Combine(lOutputPath, "Output - Dim Gas.csv"));
            lDimGasCsv.WriteLine("0;C");
            lDimGasCsv.WriteLine("1;CH4");
            lDimGasCsv.WriteLine("2;N2O");
            lDimGasCsv.WriteLine("3;SF6");
            lDimGasCsv.Close();

            lDimRegionCsv = new StreamWriter(Path.Combine(lOutputPath, "Output - Dim Region.csv"));
            for (int i = 1; i <= LegacyConstants.NoReg; i++)
                lDimRegionCsv.WriteLine(i.ToString() + ";" + LegacyRegionNames.GetName(i) + "");
            lDimRegionCsv.Close();

            lDimSectorCsv = new StreamWriter(Path.Combine(lOutputPath, "Output - Dim Sector.csv"));
            lDimSectorCsv.WriteLine("0;eloss;Water");
            lDimSectorCsv.WriteLine("1;eloss;Forests");
            lDimSectorCsv.WriteLine("2;eloss;Heating");
            lDimSectorCsv.WriteLine("3;eloss;Cooling");
            lDimSectorCsv.WriteLine("4;eloss;Agriculture");
            lDimSectorCsv.WriteLine("5;eloss;Dryland");
            lDimSectorCsv.WriteLine("6;eloss;SeaProtection");
            lDimSectorCsv.WriteLine("7;eloss;Imigration");
            lDimSectorCsv.WriteLine("8;sloss;Species");
            lDimSectorCsv.WriteLine("9;sloss;Death");
            lDimSectorCsv.WriteLine("10;sloss;Morbidity");
            lDimSectorCsv.WriteLine("11;sloss;Wetland");
            lDimSectorCsv.WriteLine("12;sloss;Emigration");
            lDimSectorCsv.WriteLine("13;eloss;Hurricane");
            lDimSectorCsv.WriteLine("14;eloss;ExtratropicalStorms");        

            lDimSectorCsv.Close();


            using (var lDimScenarioCsv = new StreamWriter(Path.Combine(lOutputPath, "Output - Dim Scenario.csv")))
            {
                foreach (Scenario lScenario in lSimulationManager.Scenarios)
                {
                    // Write Scenario dimension file
                    lDimScenarioCsv.Write(lScenario.Id);
                    lDimScenarioCsv.Write(";");
                    lDimScenarioCsv.Write(lScenario.Name);
                    lDimScenarioCsv.WriteLine();
                }
            }


            lDimYearCsv = new StreamWriter(Path.Combine(lOutputPath, "Output - Dim Year.csv"));
            for (int i = 1950; i <= 2300; i++)
            {
                lYearStr = i.ToString();
                lDimYearCsv.Write(lYearStr);
                lDimYearCsv.Write(";");
                lDimYearCsv.Write(lYearStr.Substring(0, 2));
                lDimYearCsv.Write("xx;");
                lDimYearCsv.Write(lYearStr.Substring(0, 3));
                lDimYearCsv.Write("x;");
                lDimYearCsv.Write(lYearStr.Substring(0, 4));
                lDimYearCsv.WriteLine();
            }
            lDimYearCsv.Close();

            var lDimEmissionYear = new List<int>();

            if (lSimulationManager.OutputInputParameters)
            {
                lGlobalInputCsv = new StreamWriter(Path.Combine(lOutputPath, "Output - Fact Parameter.csv"));
                lRegionInputCsv = new StreamWriter(Path.Combine(lOutputPath, "Output - Fact ParameterRegion.csv"));
                lYearInputCsv = new StreamWriter(Path.Combine(lOutputPath, "Output - Fact ParameterYear.csv"));
                lRegionYearInputCsv = new StreamWriter(Path.Combine(lOutputPath, "Output - Fact ParameterYearRegion.csv"));
                lParameterIdCsv = new StreamWriter(Path.Combine(lOutputPath, "Output - Dim Parameter.csv"));
            }

            foreach (Run lRun in lSimulationManager.Runs)
            {
                if (lSimulationManager.SameRandomStreamPerRun)
                    lRandom = GetNewRandom(lSimulationManager);

                lParam = new FundParameter(LegacyConstants.NYear, LegacyConstants.NoReg, lRandom);
                foreach (string filename in lRun.Scenario.ExcelFiles)
                    lParam.ReadExcelFile(filename);

                Console.WriteLine(lRun.Scenario.Name);
                // Set defaults
                lRuns = lRun.MonteCarloRuns;

                if (lRun.Mode == RunMode.MarginalRun)
                {
                    // DA: Use MargMain for marginal cost, use Main for total cost and optimisation
                    // modes
                    lMarginalRun = new TMarginalRun(lRun, MarginalGas.C, lRun.EmissionYear, lOutputPath, lParam);

                    if (!lDimEmissionYear.Contains(lRun.EmissionYear))
                        lDimEmissionYear.Add(lRun.EmissionYear);

                    lMarginalRun.AggregateDamageCsv = lAggregateMarginalDamage;

                    if (lRun.OutputDisaggregatedData)
                    {
                        lMarginalRun.YearRegionCsv = gDisaggregatedYearRegionCsvFile;
                        lMarginalRun.YearRegionWeightingSchemeCsv = gDisaggregatedYearRegionWeightingschemeCsvFile;
                        lMarginalRun.YearRegionSectorWeightingSchemeCsv = gDisaggregatedCsvFile;
                    }

                    if (lSimulationManager.OutputInputParameters)
                    {
                        lMarginalRun.GlobalInputCsv = lGlobalInputCsv;
                        lMarginalRun.RegionInputCsv = lRegionInputCsv;
                        lMarginalRun.YearInputCsv = lYearInputCsv;
                        lMarginalRun.RegionYearInputCsv = lRegionYearInputCsv;
                    }

                    var watch = new System.Diagnostics.Stopwatch();
                    watch.Start();
                    lMarginalRun.Run();
                    watch.Stop();
                    //Console.WriteLine("Elapsed time: {0}", watch.Elapsed);
                }
                else if (lRun.Mode == RunMode.EpaEMF22AvgMarginalRun)
                {
                    // DA: Use MargMain for marginal cost, use Main for total cost and optimisation
                    // modes
                    lMarginalRun = new TMarginalRun(lRun, MarginalGas.C, lRun.EmissionYear, lOutputPath, lParam);

                    double initialTax = lRun.InitialTax;

                    lMarginalRun.AdditionalInitCode = delegate(FundWorkflow fw)
                    {
                        var fw2 = fw;
                        double?[] taxCache = new double?[3900];

                        fw.ModelFields.Set2DimensionalParameter<Esmf.Timestep, Fund.CommonDimensions.Region, double>("emissions", "currtax",
                            (Esmf.Timestep t, Fund.CommonDimensions.Region r) =>
                            {
                                if (t < Timestep.FromYear(2010))
                                    return 0.0;
                                else if (t == Timestep.FromYear(2010))
                                    return initialTax;
                                else if (t > Timestep.FromYear(2300))
                                    return 0.0;
                                else
                                {
                                    if (taxCache[t.Value] == null)
                                    {
                                        if (fw2.ClimateState.radforc[t - 1] < fw2.ClimateState.radforc[t - 2])
                                            taxCache[t.Value] = fw2.EmissionsState.currtax[t - 1, r] / 1.05;
                                        else if (fw2.ClimateState.mco2[t - 1] < fw2.ClimateState.mco2[Timestep.FromYear(1990)] * 0.1)
                                            taxCache[t.Value] = fw2.EmissionsState.currtax[t - 1, r];
                                        else
                                            taxCache[t.Value] = fw2.EmissionsState.currtax[t - 1, r] * 1.05;
                                    }

                                    return (double)taxCache[t.Value];
                                }
                            });

                        fw.ModelFields.Set2DimensionalParameter<Esmf.Timestep, Fund.CommonDimensions.Region, double>("emissions", "currtaxch4",
                            (Esmf.Timestep t, Fund.CommonDimensions.Region r) =>
                            {
                                if (t < Timestep.FromYear(2010))
                                    return 0.0;
                                else if (t == Timestep.FromYear(2010))
                                    return initialTax;
                                else if (t > Timestep.FromYear(2300))
                                    return 0.0;
                                else
                                {
                                    if (taxCache[t.Value] == null)
                                    {
                                        if (fw2.ClimateState.radforc[t - 1] < fw2.ClimateState.radforc[t - 2])
                                            taxCache[t.Value] = fw2.EmissionsState.currtaxch4[t - 1, r] / 1.05;
                                        else if (fw2.ClimateState.mco2[t - 1] < fw2.ClimateState.mco2[Timestep.FromYear(1990)] * 0.1)
                                            taxCache[t.Value] = fw2.EmissionsState.currtaxch4[t - 1, r];
                                        else
                                            taxCache[t.Value] = fw2.EmissionsState.currtaxch4[t - 1, r] * 1.05;
                                    }

                                    return (double)taxCache[t.Value];

                                }
                            });

                        fw.ModelFields.Set2DimensionalParameter<Esmf.Timestep, Fund.CommonDimensions.Region, double>("emissions", "currtaxn2o",
                            (Esmf.Timestep t, Fund.CommonDimensions.Region r) =>
                            {
                                if (t < Timestep.FromYear(2010))
                                    return 0.0;
                                else if (t == Timestep.FromYear(2010))
                                    return initialTax;
                                else if (t > Timestep.FromYear(2300))
                                    return 0.0;
                                else
                                {
                                    if (taxCache[t.Value] == null)
                                    {
                                        if (fw2.ClimateState.radforc[t - 1] < fw2.ClimateState.radforc[t - 2])
                                            taxCache[t.Value] = fw2.EmissionsState.currtaxn2o[t - 1, r] / 1.05;
                                        else if (fw2.ClimateState.mco2[t - 1] < fw2.ClimateState.mco2[Timestep.FromYear(1990)] * 0.1)
                                            taxCache[t.Value] = fw2.EmissionsState.currtaxn2o[t - 1, r];
                                        else
                                            taxCache[t.Value] = fw2.EmissionsState.currtaxn2o[t - 1, r] * 1.05;
                                    }

                                    return (double)taxCache[t.Value];

                                }
                            });
                    };

                    if (!lDimEmissionYear.Contains(lRun.EmissionYear))
                        lDimEmissionYear.Add(lRun.EmissionYear);

                    lMarginalRun.AggregateDamageCsv = lAggregateMarginalDamage;

                    if (lRun.OutputDisaggregatedData)
                    {
                        lMarginalRun.YearRegionCsv = gDisaggregatedYearRegionCsvFile;
                        lMarginalRun.YearRegionWeightingSchemeCsv = gDisaggregatedYearRegionWeightingschemeCsvFile;
                        lMarginalRun.YearRegionSectorWeightingSchemeCsv = gDisaggregatedCsvFile;
                    }

                    if (lSimulationManager.OutputInputParameters)
                    {
                        lMarginalRun.GlobalInputCsv = lGlobalInputCsv;
                        lMarginalRun.RegionInputCsv = lRegionInputCsv;
                        lMarginalRun.YearInputCsv = lYearInputCsv;
                        lMarginalRun.RegionYearInputCsv = lRegionYearInputCsv;
                    }

                    var watch = new System.Diagnostics.Stopwatch();
                    watch.Start();
                    lMarginalRun.Run();
                    watch.Stop();
                    //Console.WriteLine("Elapsed time: {0}", watch.Elapsed);          
                }
                else if (lRun.Mode == RunMode.OptimalTaxation)
                {
                    throw new NotImplementedException();
                }
                else if (lRun.Mode == RunMode.MarginalRun2)
                {
                    throw new NotImplementedException();
                }
                else if (lRun.Mode == RunMode.FullMarginalRun)
                {
                    MarginalGas[] gases = { MarginalGas.C };
                    foreach (MarginalGas gas in gases)
                    {
                        for (int emissionyear = 2010; emissionyear <= 2100; emissionyear += 5)
                        {
                            if (!lDimEmissionYear.Contains(emissionyear))
                                lDimEmissionYear.Add(emissionyear);

                            Console.WriteLine("Now doing year {0} and gas {1}", emissionyear, gas);
                            // DA: Use MargMain for marginal cost, use Main for total cost and optimisation
                            // modes
                            lMarginalRun = new TMarginalRun(lRun, gas, emissionyear, lOutputPath, lParam);

                            lMarginalRun.AggregateDamageCsv = lAggregateMarginalDamage;

                            if (lRun.OutputDisaggregatedData)
                            {
                                lMarginalRun.YearRegionCsv = gDisaggregatedYearRegionCsvFile;
                                lMarginalRun.YearRegionWeightingSchemeCsv = gDisaggregatedYearRegionWeightingschemeCsvFile;
                                lMarginalRun.YearRegionSectorWeightingSchemeCsv = gDisaggregatedCsvFile;
                            }

                            if (lSimulationManager.OutputInputParameters)
                            {
                                lMarginalRun.GlobalInputCsv = lGlobalInputCsv;
                                lMarginalRun.RegionInputCsv = lRegionInputCsv;
                                lMarginalRun.YearInputCsv = lYearInputCsv;
                                lMarginalRun.RegionYearInputCsv = lRegionYearInputCsv;
                            }

                            var watch = new System.Diagnostics.Stopwatch();
                            watch.Start();
                            lMarginalRun.Run();
                            watch.Stop();
                            //Console.WriteLine("Elapsed time: {0}", watch.Elapsed);
                        }
                    }
                }
                else if (lRun.Mode == RunMode.EpaMarginalRun)
                {
                    int[] emissionyears = { 2010, 2020, 2030, 2040, 2050 };
                    MarginalGas[] gases = { MarginalGas.C };

                    foreach (MarginalGas gas in gases)
                    {
                        foreach (int emissionyear in emissionyears)
                        {
                            if (!lDimEmissionYear.Contains(emissionyear))
                                lDimEmissionYear.Add(emissionyear);

                            Console.WriteLine("Now doing year {0} and gas {1}", emissionyear, gas);
                            // DA: Use MargMain for marginal cost, use Main for total cost and optimisation
                            // modes
                            lMarginalRun = new TMarginalRun(lRun, gas, emissionyear, lOutputPath, lParam);

                            lMarginalRun.AggregateDamageCsv = lAggregateMarginalDamage;

                            if (lRun.OutputDisaggregatedData)
                            {
                                lMarginalRun.YearRegionCsv = gDisaggregatedYearRegionCsvFile;
                                lMarginalRun.YearRegionWeightingSchemeCsv = gDisaggregatedYearRegionWeightingschemeCsvFile;
                                lMarginalRun.YearRegionSectorWeightingSchemeCsv = gDisaggregatedCsvFile;
                            }

                            if (lSimulationManager.OutputInputParameters)
                            {
                                lMarginalRun.GlobalInputCsv = lGlobalInputCsv;
                                lMarginalRun.RegionInputCsv = lRegionInputCsv;
                                lMarginalRun.YearInputCsv = lYearInputCsv;
                                lMarginalRun.RegionYearInputCsv = lRegionYearInputCsv;
                            }

                            var watch = new System.Diagnostics.Stopwatch();
                            watch.Start();
                            lMarginalRun.Run();
                            watch.Stop();
                            //Console.WriteLine("Elapsed time: {0}", watch.Elapsed);
                        }
                    }
                }
                else if (lRun.Mode == RunMode.TotalRun)
                {
                    // DA: Use MargMain for marginal cost, use Main for total cost and optimisation
                    // modes
                    var lTotalDamageRun = new TotalDamage(lRun, lOutputPath, lParam);

                    lTotalDamageRun.AggregateDamageCsv = lAggregateMarginalDamage;

                    if (lRun.OutputDisaggregatedData)
                    {
                        lTotalDamageRun.YearRegionCsv = gDisaggregatedYearRegionCsvFile;
                        lTotalDamageRun.YearRegionWeightingSchemeCsv = gDisaggregatedYearRegionWeightingschemeCsvFile;
                        lTotalDamageRun.YearRegionSectorWeightingSchemeCsv = gDisaggregatedCsvFile;
                    }

                    if (lSimulationManager.OutputInputParameters)
                    {
                        lTotalDamageRun.GlobalInputCsv = lGlobalInputCsv;
                        lTotalDamageRun.RegionInputCsv = lRegionInputCsv;
                        lTotalDamageRun.YearInputCsv = lYearInputCsv;
                        lTotalDamageRun.RegionYearInputCsv = lRegionYearInputCsv;
                    }

                    lTotalDamageRun.Run();

                }
            }

            lAggregateMarginalDamage.Close();
            gDisaggregatedYearCsvFile.Close();
            TempOutputFile.Close();
            gBGEResultsCSVFile.Close();

            using (var lDimEmissionYearCsv = new StreamWriter(Path.Combine(lOutputPath, "Output - Dim Emissionyear.csv")))
            {
                foreach (int emissionyear in lDimEmissionYear)
                    lDimEmissionYearCsv.WriteLine("{0};{1}", emissionyear, emissionyear);
            }

            if (lSimulationManager.Runs.Exists((Run run) => run.OutputDisaggregatedData))
            {
                gDisaggregatedCsvFile.Close();
                gDisaggregatedYearRegionCsvFile.Close();
                gDisaggregatedYearRegionWeightingschemeCsvFile.Close();
            }

            if (lSimulationManager.OutputInputParameters)
            {
                lGlobalInputCsv.Close();
                lRegionInputCsv.Close();
                lYearInputCsv.Close();
                lRegionYearInputCsv.Close();


                lParameterIdCsv.Close();
            }

        }

        private static Random GetNewRandom(SimulationManager lSimulationManager)
        {
            Random lRandom;
            switch (lSimulationManager.Rng)
            {
                case RandomNumberGenerator.DotNet:
                    if (lSimulationManager.Randomize)
                        lRandom = new Random();
                    else
                        lRandom = new Random(34514325);
                    break;
                case RandomNumberGenerator.MersenneTwister:
                    if (lSimulationManager.Randomize)
                        throw new Exception("Support for randomization is not there for mersenne twister");
                    else
                        lRandom = new jp.takel.PseudoRandom.MersenneTwister();
                    break;
                default: throw new Exception("You specified a RNG that is not implemented");
            }
            return lRandom;
        }

        public static void UnhandledExceptionHandler(object sender, UnhandledExceptionEventArgs args)
        {
            var e = (Exception)args.ExceptionObject;

            Console.WriteLine("An unhandled exception occurred that is handled by the the ConsoleApp.UnhandledExceptionHandler:");
            Console.WriteLine();
            Console.WriteLine(e.GetType().Name);
            Console.WriteLine(e.Message);
            Console.WriteLine(e.Source);
            Console.WriteLine(e.StackTrace);
        }
    }

}