﻿// Climate Framework for Uncertainty, Negotiation and Distribution
// Version 3.5
// (c) David Anthoff and Richard S.J. Tol
// http://www.fund-model.org
using Esmf;
using Fund.CommonDimensions;
using System;

namespace Fund
{

    public interface ISocioEconomic2Welfare
    {
        IParameter2Dimensional<Timestep, Fund.CommonDimensions.Region, double> population { get; }
        IParameter2Dimensional<Timestep, Fund.CommonDimensions.Region, double> consumption { get; }

        IVariable1Dimensional<Timestep, double> totalpopulation { get; }
        IVariable1Dimensional<Timestep, double> totalconsumption { get; }
    }

    public class SocioEconomic2Welfare
    {

        public void Run(Clock clock, ISocioEconomic2Welfare state)
        {
            var s = state;
            var t = clock.Current;

            var totalpopulation = 0.0;
            var totalconsumption = 0.0;

            foreach (Region r in Enum.GetValues(typeof(Region)))
            {
                totalpopulation = totalpopulation + s.population[t, r];
                totalconsumption = totalconsumption + s.consumption[t, r];
            }

            s.totalpopulation[t] = totalpopulation * 1000000;
            s.totalconsumption[t] = totalconsumption;
        }
    }

}