﻿// Climate Framework for Uncertainty, Negotiation and Distribution
// Version 3.5
// (c) David Anthoff and Richard S.J. Tol
// http://www.fund-model.org
using System.Collections.Generic;
using System.Reflection.Emit;
using Esmf;
using System;

namespace Fund
{

    public class RegionParameterConnector<D1> : Esmf.IParameter1Dimensional<D1, double>
    {
        private double[] _values;
        private Enum2IntConversionDelegate<D1> _enumConversionDelegate;
        public double this[D1 d] { get { return GetItem(d); } }

        public double GetItem(D1 d)
        {
            return _values[_enumConversionDelegate(d) - 1];
        }

        public RegionParameterConnector(Parameters param, string name)
        {
            if (typeof(D1) == typeof(Esmf.Timestep))
                throw new InvalidOperationException();
            else if (typeof(D1).BaseType == typeof(System.Enum))
            {
                var values = (int[])Enum.GetValues(typeof(D1));

                _values = new Double[16];

                foreach (int r in values)
                    _values[r - 1] = param.GetItemByKey(new RegionParameterKey(name, Enum.GetName(typeof(D1), r)));

                _enumConversionDelegate = Enum2IntConversionSingleton<D1>.Instance.Delegate;
            }
            else
                throw new InvalidOperationException();

        }

        public System.Collections.Generic.IEnumerable<Esmf.Parameter1DimensionalMember<D1, System.Double>> GetEnumerator()
        {
            throw new NotImplementedException();
        }
    }
}