﻿// Climate Framework for Uncertainty, Negotiation and Distribution
// Version 3.5
// (c) David Anthoff and Richard S.J. Tol
// http://www.fund-model.org
using System.Configuration;
using System.IO;
using System;

namespace Fund
{

    public class TotalDamage
    {
        private TextWriter m_YearRegionCsv;
        private TextWriter m_YearRegionWeightingschemeCsv;
        private TextWriter m_YearRegionSectorWeightingSchemeCsv;
        private TextWriter m_AggregateDamageCsv;
        private TextWriter m_GlobalInputCsv;
        private TextWriter m_RegionInputCsv;
        private TextWriter m_YearInputCsv;
        private TextWriter m_RegionYearInputCsv;

        private Run _run;
        private string _outputPath;
        private FundParameter _parameters;

        public TextWriter AggregateDamageCsv { get { return m_AggregateDamageCsv; } set { m_AggregateDamageCsv = value; } }
        public TextWriter YearRegionCsv { get { return m_YearRegionCsv; } set { m_YearRegionCsv = value; } }
        public TextWriter YearRegionWeightingSchemeCsv { get { return m_YearRegionWeightingschemeCsv; } set { m_YearRegionWeightingschemeCsv = value; } }
        public TextWriter YearRegionSectorWeightingSchemeCsv { get { return m_YearRegionSectorWeightingSchemeCsv; } set { m_YearRegionSectorWeightingSchemeCsv = value; } }
        public TextWriter GlobalInputCsv { get { return m_GlobalInputCsv; } set { m_GlobalInputCsv = value; } }
        public TextWriter RegionInputCsv { get { return m_RegionInputCsv; } set { m_RegionInputCsv = value; } }
        public TextWriter YearInputCsv { get { return m_YearInputCsv; } set { m_YearInputCsv = value; } }
        public TextWriter RegionYearInputCsv { get { return m_RegionYearInputCsv; } set { m_RegionYearInputCsv = value; } }

        public void WriteAggregateDamage(int RunId, int WeightingschemeId, double Damage, WeightingCombination[] WeightingCombinations)
        {
            if (m_AggregateDamageCsv != null)
            {
                m_AggregateDamageCsv.Write(_run.Scenario.Id);
                m_AggregateDamageCsv.Write(";");

                if (_run.OutputVerbal)
                {
                    switch (Convert.ToInt32(_parameters.marginalGas))
                    {
                        case 0: m_AggregateDamageCsv.Write("\"C\""); break;
                        default: throw new NotImplementedException("Only carbon supported for output at this point");
                    }
                }
                else
                    m_AggregateDamageCsv.Write(Convert.ToInt32(_parameters.marginalGas));

                m_AggregateDamageCsv.Write(";");
                m_AggregateDamageCsv.Write(Convert.ToInt32(_parameters.emissionPeriod));
                m_AggregateDamageCsv.Write(";");

                if (_run.OutputVerbal)
                {
                    if (RunId == 0)
                        m_AggregateDamageCsv.Write("\"Best guess\"");
                    else
                        m_AggregateDamageCsv.Write(RunId);
                }
                else
                    m_AggregateDamageCsv.Write(RunId);

                m_AggregateDamageCsv.Write(";");

                if (_run.OutputVerbal)
                    m_AggregateDamageCsv.Write("\"" + WeightingCombinations[WeightingschemeId].Name + "\"");
                else
                    m_AggregateDamageCsv.Write(WeightingschemeId);

                m_AggregateDamageCsv.Write(";");
                m_AggregateDamageCsv.Write("{0:f15}", Damage);
                m_AggregateDamageCsv.WriteLine();

            }
        }

        public void WriteDamage(int RunId, Damage i_Damage, int WeightingschemeId, double Weight, WeightingCombination[] WeightingCombinations)
        {
            if (m_YearRegionSectorWeightingSchemeCsv != null)
            {
                m_YearRegionSectorWeightingSchemeCsv.Write(_run.Scenario.Id);
                m_YearRegionSectorWeightingSchemeCsv.Write(";");

                if (_run.OutputVerbal)
                {
                    if (RunId == 0)
                        m_YearRegionSectorWeightingSchemeCsv.Write("\"Best guess\"");
                    else
                        m_YearRegionSectorWeightingSchemeCsv.Write(RunId);
                }
                else
                    m_YearRegionSectorWeightingSchemeCsv.Write(RunId);

                // Marginal Gas and emissions year are empty for total runs
                m_YearRegionSectorWeightingSchemeCsv.Write(";;");

                m_YearRegionSectorWeightingSchemeCsv.Write(";");
                m_YearRegionSectorWeightingSchemeCsv.Write((i_Damage.Year + 1950));
                m_YearRegionSectorWeightingSchemeCsv.Write(";");

                if (_run.OutputVerbal)
                    m_YearRegionSectorWeightingSchemeCsv.Write(LegacyRegionNames.GetName(i_Damage.Region));
                else
                    m_YearRegionSectorWeightingSchemeCsv.Write(i_Damage.Region);

                m_YearRegionSectorWeightingSchemeCsv.Write(";");

                if (_run.OutputVerbal)
                    m_YearRegionSectorWeightingSchemeCsv.Write(Enum.GetName(typeof(Sector), i_Damage.Sector));
                else
                    m_YearRegionSectorWeightingSchemeCsv.Write((int)i_Damage.Sector);

                m_YearRegionSectorWeightingSchemeCsv.Write(";");

                if (_run.OutputVerbal)
                    m_YearRegionSectorWeightingSchemeCsv.Write("\"" + WeightingCombinations[WeightingschemeId].Name + "\"");
                else
                    m_YearRegionSectorWeightingSchemeCsv.Write(WeightingschemeId);

                m_YearRegionSectorWeightingSchemeCsv.Write(";");
                double i_e = i_Damage.DamageValue * Weight;
                m_YearRegionSectorWeightingSchemeCsv.Write("{0:f15}", i_e);
                m_YearRegionSectorWeightingSchemeCsv.WriteLine();
            }
        }

        public void WriteYearRegionWeightingschemeData(int RunId, int RegionId, int WeightingSchemeId, WeightingCombination[] i_weightingCombinations, int Year)
        {
            if (m_YearRegionWeightingschemeCsv != null)
            {
                m_YearRegionWeightingschemeCsv.Write(_run.Scenario.Id);
                m_YearRegionWeightingschemeCsv.Write(";");
                m_YearRegionWeightingschemeCsv.Write(Convert.ToInt32(_parameters.marginalGas));
                m_YearRegionWeightingschemeCsv.Write(";");
                m_YearRegionWeightingschemeCsv.Write(Convert.ToInt32(_parameters.emissionPeriod));
                m_YearRegionWeightingschemeCsv.Write(";");
                m_YearRegionWeightingschemeCsv.Write(RunId);
                m_YearRegionWeightingschemeCsv.Write(";");
                m_YearRegionWeightingschemeCsv.Write((Year + 1950));
                m_YearRegionWeightingschemeCsv.Write(";");
                m_YearRegionWeightingschemeCsv.Write(RegionId);
                m_YearRegionWeightingschemeCsv.Write(";");
                m_YearRegionWeightingschemeCsv.Write(WeightingSchemeId);
                m_YearRegionWeightingschemeCsv.Write(";");
                double i_e = i_weightingCombinations[WeightingSchemeId][Year, RegionId];
                m_YearRegionWeightingschemeCsv.Write("{0:f15}", i_e);
                m_YearRegionWeightingschemeCsv.WriteLine();
            }
        }

        public void WriteYearRegionData(int RunId, int Year, int Region, double Income, double Population)
        {
            if (m_YearRegionCsv != null)
            {
                m_YearRegionCsv.Write(_run.Scenario.Id);
                m_YearRegionCsv.Write(";");
                m_YearRegionCsv.Write(Convert.ToInt32(_parameters.marginalGas));
                m_YearRegionCsv.Write(";");
                m_YearRegionCsv.Write(Convert.ToInt32(_parameters.emissionPeriod));
                m_YearRegionCsv.Write(";");
                m_YearRegionCsv.Write(RunId);
                m_YearRegionCsv.Write(";");
                m_YearRegionCsv.Write((Year + 1950));
                m_YearRegionCsv.Write(";");
                m_YearRegionCsv.Write(Region);
                m_YearRegionCsv.Write(";");
                m_YearRegionCsv.Write("{0:f15}", Income);
                m_YearRegionCsv.Write(";");
                m_YearRegionCsv.Write("{0:f15}", Population);
                m_YearRegionCsv.WriteLine();
            }
        }

        public TotalDamage(Run runData, string outputPath, FundParameter parameters)
        {
            _parameters = parameters;
            _run = runData;
            _outputPath = outputPath;
        }

        public void Run()
        {
            WeightingCombination[] i_weightingCombinations;
            TextWriter lDimWeightingschemeCsv;

            _parameters.SetToBestGuessValue();

            Fund28LegacyWeightingCombinations.GetWeightingCombinationsFromName(_run.WeightingCombination, out i_weightingCombinations, _parameters);

            lDimWeightingschemeCsv = new StreamWriter(Path.Combine(_outputPath, "Output - Dim Weightingscheme.csv"));
            for (int i = 0; i < i_weightingCombinations.Length; i++)
                lDimWeightingschemeCsv.WriteLine(i.ToString() + ";" + i_weightingCombinations[i].Name);

            lDimWeightingschemeCsv.Close();

            for (int i = 0; i <= _run.MonteCarloRuns; i++)
            {
                if (i == 0)
                {
                    Console.WriteLine("Best guess run ");

                    _parameters.SetToBestGuessValue();
                    _parameters.WriteValuesToCsv(_run.Scenario.Id, i,
                         m_GlobalInputCsv, m_RegionInputCsv, m_YearInputCsv, m_RegionYearInputCsv);
                }
                else
                {
                    Console.WriteLine("Run " + i.ToString());

                    _parameters.SetToRandomValue();
                    _parameters.WriteValuesToCsv(_run.Scenario.Id, i,
                         m_GlobalInputCsv, m_RegionInputCsv, m_YearInputCsv, m_RegionYearInputCsv);
                }

                DoOneRun(i, i_weightingCombinations);
            }
        }

        public void DoOneRun(int RunId, WeightingCombination[] i_weightingCombinations)
        {
            Damage i_Damage;
            double i_aggregatedDamage;
            ModelOutput i_output1;

            // Create Output object for run 1, set addmp to 0 so that
            // the extra greenhouse gases are not emitted and then run
            // the model
            i_output1 = new ModelOutput();
            MargRun(i_output1);

            for (int i = 0; i < i_weightingCombinations.Length; i++)
            {
                i_weightingCombinations[i].CalculateWeights(i_output1);
                i_aggregatedDamage = i_weightingCombinations[i].AddDamagesUp(i_output1.Damages, _run.YearsToAggregate, Convert.ToInt32(_parameters.emissionPeriod));

                WriteAggregateDamage(RunId, i, i_aggregatedDamage, i_weightingCombinations);

            }

            for (int l = 0; l < i_output1.Damages.Count; l++)
            {
                i_Damage = i_output1.Damages[l];
                if ((i_Damage.Year >= _parameters.emissionPeriod - 1950) && (i_Damage.Year < _parameters.emissionPeriod - 1950 + _run.YearsToAggregate))
                {
                    for (int k = 0; k < i_weightingCombinations.Length; k++)
                        WriteDamage(RunId, i_Damage, k, i_weightingCombinations[k][i_Damage.Year, i_Damage.Region], i_weightingCombinations);
                }
            }
            for (int l = Convert.ToInt32(_parameters.emissionPeriod - 1950); l <= (Convert.ToInt32(_parameters.emissionPeriod - 1950) - 1950 + _run.YearsToAggregate); l++)
            {
                for (int k = 1; k <= LegacyConstants.NoReg; k++)
                {
                    WriteYearRegionData(RunId, l, k, i_output1.Incomes[l, k], i_output1.Populations[l, k]);
                    for (int m = 0; m < i_weightingCombinations.Length; m++)
                        WriteYearRegionWeightingschemeData(RunId, k, m, i_weightingCombinations, l);
                }
            }
        }

        public void MargRun(ModelOutput Output)
        {
            var fundWorkflow = new FundWorkflow(_parameters);

            fundWorkflow.Run(Output);
        }
    }
}