﻿// Climate Framework for Uncertainty, Negotiation and Distribution
// Version 3.5
// (c) David Anthoff and Richard S.J. Tol
// http://www.fund-model.org
using System.Collections.Generic;
using System;
using Esmf;

namespace Fund
{

    public static class Fund28LegacyWeightingCombinations
    {
        public static void GetWeightingCombinationsFromName(string name, out WeightingCombination[] i_WeightingCombinations, FundParameter parameters)
        {
            switch (name)
            {
                case "Standard": StandardWeightingCombinations(out i_WeightingCombinations, parameters); break;
                case "Raw": RawWeightingCombinations(out i_WeightingCombinations, parameters); break;
                default: throw new ApplicationException("Invalid weighting scheme");
            }
        }

        private static void StandardWeightingCombinations(out WeightingCombination[] WeightingCombinations, FundParameter parameters)
        {
            WeightingCombination[] i_weightingCombinations = new WeightingCombination[7];

            i_weightingCombinations[0] = new WeightingCombination();
            i_weightingCombinations[0].Add(new ConstantDiscounting(0.00));

            i_weightingCombinations[1] = new WeightingCombination();
            i_weightingCombinations[1].Add(new ConstantDiscounting(0.00));
            i_weightingCombinations[1].Add(new AverageEquityWeighting());

            i_weightingCombinations[2] = new WeightingCombination();
            i_weightingCombinations[2].Add(new ConstantDiscounting(0.01));

            i_weightingCombinations[3] = new WeightingCombination();
            i_weightingCombinations[3].Add(new ConstantDiscounting(0.01));
            i_weightingCombinations[3].Add(new AverageEquityWeighting());

            i_weightingCombinations[4] = new WeightingCombination();
            i_weightingCombinations[4].Add(new ConstantDiscounting(0.03));

            i_weightingCombinations[5] = new WeightingCombination();
            i_weightingCombinations[5].Add(new ConstantDiscounting(0.03));
            i_weightingCombinations[5].Add(new AverageEquityWeighting());

            i_weightingCombinations[6] = new WeightingCombination();
            i_weightingCombinations[6].Add(new GreenbookDiscounting(50));

            WeightingCombinations = i_weightingCombinations;
        }

        private static void RawWeightingCombinations(out WeightingCombination[] WeightingCombinations, FundParameter parameters)
        {
            var i_weightingCombinations = new List<WeightingCombination>();

            WeightingCombination w;

            w = new WeightingCombination("RAW");
            w.Add(new UnityWeight());
            i_weightingCombinations.Add(w);

            WeightingCombinations = i_weightingCombinations.ToArray();
        }

    }
}
