﻿// Climate Framework for Uncertainty, Negotiation and Distribution
// Version 3.5
// (c) David Anthoff and Richard S.J. Tol
// http://www.fund-model.org
using System;

namespace Fund
{

    public class AnthoffEquityWeighting : WeightingScheme
    {
        private int m_baseYear;
        private int m_baseRegion;
        private double m_inequalityAversion;

        public override void CalculateWeights(ModelOutput ModelOutput)
        {
            double i_totalIncome, i_totalPopulation;
            double i_baseYpC;
            double i_YpC;

            if (m_baseRegion == -1)
            {
                i_totalIncome = 0;
                i_totalPopulation = 0;
                for (int i = 1; i <= LegacyConstants.NoReg; i++)
                {
                    i_totalIncome = i_totalIncome + ModelOutput.Incomes[m_baseYear, i];
                    i_totalPopulation = i_totalPopulation + ModelOutput.Populations[m_baseYear, i];
                }
                i_baseYpC = i_totalIncome / i_totalPopulation;
            }
            else
                i_baseYpC = ModelOutput.Incomes[m_baseYear, m_baseRegion] / ModelOutput.Populations[m_baseYear, m_baseRegion];

            for (int i = 1; i <= LegacyConstants.NoReg; i++)
            {
                // TODO -oDavid Anthoff: Change start of "real" period to constant
                for (int l = 50; l <= LegacyConstants.NYear; l++)
                {
                    i_YpC = ModelOutput.Incomes[l, i] / ModelOutput.Populations[l, i];
                    AddWeight(l, i, Math.Pow(i_baseYpC / i_YpC, m_inequalityAversion));
                }
            }
        }

        public AnthoffEquityWeighting(int BaseYear, string BaseRegion, double InequalityAversion)
            : base()
        {
            int i_regionIndex;

            i_regionIndex = -1;

            for (int i = 1; i <= LegacyConstants.NoReg; i++)
            {
                if (LegacyRegionNames.GetName(i) == BaseRegion)
                    i_regionIndex = i;
            }

            m_baseYear = BaseYear;
            m_baseRegion = i_regionIndex;
            m_inequalityAversion = InequalityAversion;
        }

        protected override string GetName()
        {
            if (m_baseRegion == -1)
                // TODO -oDavid Anthoff: Exchange 1950 with constant
                return "utility (in " + Convert.ToString(m_baseYear + 1950) + " units, mu=" + Convert.ToString(m_inequalityAversion) + ")";
            else
                return "utility (in " + Convert.ToString(m_baseYear + 1950) + ", " + LegacyRegionNames.GetName(m_baseRegion) + " units, mu=" + Convert.ToString(m_inequalityAversion) + ")";
        }
    }
}