﻿// Climate Framework for Uncertainty, Negotiation and Distribution
// Version 3.5
// (c) David Anthoff and Richard S.J. Tol
// http://www.fund-model.org
using System;

namespace Fund
{

    public class ConstantDiscountRate : WeightingScheme
    {
        private double m_rate;
        private string m_name;
        private int m_baseYear;

        protected override string GetName()
        {
            return string.Format("{0}={1}%", m_name, m_rate * 100);
        }

        protected override double GetWeight(int Year, int Region)
        {
            if (Year < m_baseYear)
                throw new Exception("this is not supported");

            return 1.0 / Math.Pow(1 + m_rate, Year - m_baseYear);
        }

        public ConstantDiscountRate(double discountRate)
            : this(discountRate, "discount rate")
        {
        }

        public override void CalculateWeights(ModelOutput ModelOutput)
        {
        }

        public ConstantDiscountRate(double discountRate, string name)
        {
            m_rate = discountRate;
            m_name = name;
            m_baseYear = 50;
        }

        public ConstantDiscountRate(double discountRate, int baseYear)
        {
            m_rate = discountRate;
            m_baseYear = baseYear;
        }
    }
}