﻿// Climate Framework for Uncertainty, Negotiation and Distribution
// Version 3.5
// (c) David Anthoff and Richard S.J. Tol
// http://www.fund-model.org
using System;

namespace Fund
{

    public class GlobalGrowthDiscounterateWeighting : WeightingScheme
    {
        private int m_baseYear;
        private double m_inequalityAversion;

        public override void CalculateWeights(ModelOutput ModelOutput)
        {
            double i_totalIncome, i_totalPopulation;
            double i_baseYpC;
            double i_YpC;

            i_totalIncome = 0;
            i_totalPopulation = 0;
            for (int i = 1; i <= LegacyConstants.NoReg; i++)
            {
                i_totalIncome = i_totalIncome + ModelOutput.Incomes[m_baseYear, i];
                i_totalPopulation = i_totalPopulation + ModelOutput.Populations[m_baseYear, i];
            }
            i_baseYpC = i_totalIncome / i_totalPopulation;

            for (int l = m_baseYear; l <= LegacyConstants.NYear; l++)
            {
                var i_income = 0.0;
                var i_population = 0.0;
                for (int i = 1; i <= LegacyConstants.NoReg; i++)
                {
                    i_income = i_income + ModelOutput.Incomes[l, i];
                    i_population = i_population + ModelOutput.Populations[l, i];
                }

                i_YpC = i_income / i_population;

                for (int i = 1; i <= LegacyConstants.NoReg; i++)
                {

                    AddWeight(l, i, Math.Pow(i_baseYpC / i_YpC, m_inequalityAversion));
                }

            }
        }

        protected override string GetName()
        {
            return "global utility (in " + Convert.ToString(m_baseYear + 1950) + " units, mu=" + Convert.ToString(m_inequalityAversion) + ")";
        }

        public GlobalGrowthDiscounterateWeighting(int BaseYear, double InequalityAversion)
        {
            m_baseYear = BaseYear;
            m_inequalityAversion = InequalityAversion;
        }
    }
}