﻿// Climate Framework for Uncertainty, Negotiation and Distribution
// Version 3.5
// (c) David Anthoff and Richard S.J. Tol
// http://www.fund-model.org
using System;

namespace Fund
{

    public class GollierUncertainGrowthDiscounting : WeightingScheme
    {
        private double m_upperBound, m_lowerBound, m_prtp;

        public override void CalculateWeights(ModelOutput ModelOutput)
        {
            double i_discountFactorUpper;
            double i_discountFactorLower;
            double i_newPerCapitaIncome, i_oldPerCapitaIncome;
            double i_perCapitaGrowthRate;

            for (int i = 1; i <= LegacyConstants.NoReg; i++)
            {
                i_discountFactorUpper = 1;
                i_discountFactorLower = 1;
                for (int l = 50; l <= LegacyConstants.NYear; l++)
                {
                    AddWeight(l, i, (i_discountFactorUpper + i_discountFactorLower) / 2);

                    i_newPerCapitaIncome = ModelOutput.Incomes[l, i] / ModelOutput.Populations[l, i];
                    i_oldPerCapitaIncome = ModelOutput.Incomes[l - 1, i] / ModelOutput.Populations[l - 1, i];
                    i_perCapitaGrowthRate = (i_newPerCapitaIncome - i_oldPerCapitaIncome) / i_oldPerCapitaIncome;


                    i_discountFactorUpper = i_discountFactorUpper / (1 + m_prtp + i_perCapitaGrowthRate + m_upperBound);
                    i_discountFactorLower = i_discountFactorLower / (1 + m_prtp + i_perCapitaGrowthRate - m_lowerBound);
                }
            }

        }

        public GollierUncertainGrowthDiscounting(double Prtp, double UpperBound, double LowerBound)
            : base()
        {
            m_prtp = Prtp;
            m_upperBound = UpperBound;
            m_lowerBound = LowerBound;
        }

        protected override string GetName()
        {
            return "Gollier (prtp=" + Convert.ToString(m_prtp * 100) + "%, boundary=" + Convert.ToString(m_upperBound * 100) + "%)";
        }

    }
}