﻿// Climate Framework for Uncertainty, Negotiation and Distribution
// Version 3.5
// (c) David Anthoff and Richard S.J. Tol
// http://www.fund-model.org
using System;

namespace Fund
{

    public class GollierUncertainGrowthDiscountingNormalDistribution : WeightingScheme
    {
        private double m_Sigma, m_prtp;

        public override void CalculateWeights(ModelOutput ModelOutput)
        {
            double i_discountFactor1;
            double i_discountFactor2;
            double i_discountFactor3;
            double i_discountFactor4;
            double i_discountFactor5;
            double i_newPerCapitaIncome, i_oldPerCapitaIncome;
            double i_perCapitaGrowthRate;

            for (int i = 1; i <= LegacyConstants.NoReg; i++)
            {
                i_discountFactor1 = 1;
                i_discountFactor2 = 1;
                i_discountFactor3 = 1;
                i_discountFactor4 = 1;
                i_discountFactor5 = 1;

                for (int l = 50; l <= LegacyConstants.NYear; l++)
                {
                    AddWeight(l, i, i_discountFactor1 * 0.0227 + i_discountFactor2 * 0.136 + i_discountFactor3 * 0.6826 + i_discountFactor4 * 0.136 + i_discountFactor5 * 0.0227);

                    i_newPerCapitaIncome = ModelOutput.Incomes[l, i] / ModelOutput.Populations[l, i];
                    i_oldPerCapitaIncome = ModelOutput.Incomes[l - 1, i] / ModelOutput.Populations[l - 1, i];
                    i_perCapitaGrowthRate = (i_newPerCapitaIncome - i_oldPerCapitaIncome) / i_oldPerCapitaIncome;

                    i_discountFactor1 = i_discountFactor1 / (1 + m_prtp + i_perCapitaGrowthRate - 2.5 * m_Sigma);
                    i_discountFactor2 = i_discountFactor2 / (1 + m_prtp + i_perCapitaGrowthRate - 1.5 * m_Sigma);
                    i_discountFactor3 = i_discountFactor3 / (1 + m_prtp + i_perCapitaGrowthRate);
                    i_discountFactor4 = i_discountFactor4 / (1 + m_prtp + i_perCapitaGrowthRate + 1.5 * m_Sigma);
                    i_discountFactor5 = i_discountFactor5 / (1 + m_prtp + i_perCapitaGrowthRate + 2.5 * m_Sigma);
                }
            }
        }

        public GollierUncertainGrowthDiscountingNormalDistribution(double Prtp, double Sigma)
            : base()
        {
            m_prtp = Prtp;
            m_Sigma = Sigma;
        }

        protected override string GetName()
        {
            return "Gollier normal (prtp=" + Convert.ToString(m_prtp * 100) + "%, sigma=" + Convert.ToString(m_Sigma * 100) + "%)";
        }
    }
}