﻿// Climate Framework for Uncertainty, Negotiation and Distribution
// Version 3.5
// (c) David Anthoff and Richard S.J. Tol
// http://www.fund-model.org
using System;

namespace Fund
{

    public class WeitzmanDiscounting : WeightingScheme
    {
        private double mMju;
        private double mSigma;

        public override void CalculateWeights(ModelOutput ModelOutput)
        {
            double i_discountFactor;
            double i_discountRate;

            i_discountFactor = 1;
            for (int i = 50; i <= LegacyConstants.NYear; i++)
            {
                for (int l = 1; l <= LegacyConstants.NoReg; l++)
                    AddWeight(i, l, i_discountFactor);

                i_discountRate = mMju / (1 + (i - 50) * (Math.Pow(mSigma, 2) / mMju));

                i_discountFactor = i_discountFactor / (1 + i_discountRate);
            }
        }

        public WeitzmanDiscounting(double Mju, double Sigma)
            : base()
        {
            mMju = Mju;
            mSigma = Sigma;
        }

        protected override string GetName()
        {
            return "Weitzman discounting";
        }
    }
}